@@ -22,9 +22,11 @@ from __future__ import unicode_literals
 import contextlib
 import datetime
 import functools
+
 from apitools.base.py import encoding
 from apitools.base.py import exceptions as api_exceptions
 from apitools.base.py import list_pager
+from googlecloudsdk.api_lib.cloudbuild import cloudbuild_util
 from googlecloudsdk.api_lib.run import condition as run_condition
 from googlecloudsdk.api_lib.run import configuration
 from googlecloudsdk.api_lib.run import domain_mapping
@@ -36,6 +38,7 @@ from googlecloudsdk.api_lib.run import service
 from googlecloudsdk.api_lib.util import apis
 from googlecloudsdk.api_lib.util import apis_internal
 from googlecloudsdk.api_lib.util import waiter
+from googlecloudsdk.command_lib.builds import submit_util
 from googlecloudsdk.command_lib.iam import iam_util
 from googlecloudsdk.command_lib.run import config_changes as config_changes_mod
 from googlecloudsdk.command_lib.run import exceptions as serverless_exceptions
@@ -180,16 +183,11 @@ class ConditionPoller(waiter.OperationPoller):
     self._dependencies = {k: set() for k in tracker}
     if dependencies is not None:
       for k in dependencies:
-        error_msg = 'Condition {} is not in the dependency tracker.'
-        if k not in tracker:
-          raise ValueError(error_msg.format(k))
-        for c in dependencies[k]:
-          if c not in tracker:
-            raise ValueError(error_msg.format(c))
-        # Add dependencies, only if they're still not complete
+        # Add dependencies, only if they're still not complete. If a stage isn't
+        # in the tracker. consider it "already complete".
         self._dependencies[k] = {
-            c for c in dependencies[k] if not tracker.IsComplete(c)
-        }
+            c for c in dependencies[k]
+            if c in tracker and not tracker.IsComplete(c)}
     self._resource_getter = resource_getter
     self._tracker = tracker
     self._resource_fail_type = exceptions.Error
@@ -271,8 +269,7 @@ class ConditionPoller(waiter.OperationPoller):
       conditions_message: str, The message from the conditions object we're
         displaying..
     """
-
-    if self._tracker.IsComplete(condition):
+    if condition not in self._tracker or self._tracker.IsComplete(condition):
       return
 
     if self._IsBlocked(condition):
@@ -303,7 +300,7 @@ class ConditionPoller(waiter.OperationPoller):
     Returns:
       bool: True if stage was completed, False if no action taken
     """
-    if self._tracker.IsComplete(condition):
+    if condition not in self._tracker or self._tracker.IsComplete(condition):
       return False
     # A blocked condition is likely to remain True (indicating the previous
     # operation concerning it was successful) until the blocking condition(s)
@@ -326,6 +323,8 @@ class ConditionPoller(waiter.OperationPoller):
     # The set of stages that aren't marked started and don't have unsatisfied
     # dependencies are newly unblocked.
     for c in self._dependencies:
+      if c not in self._tracker:
+        continue
       if self._tracker.IsWaiting(c) and not self._IsBlocked(c):
         self._tracker.StartStage(c)
     # TODO(b/120679874): Should not have to manually call Tick()
@@ -342,7 +341,7 @@ class ConditionPoller(waiter.OperationPoller):
       DeploymentFailedError: If the 'Ready' condition failed.
     """
     # Don't fail an already failed stage.
-    if self._tracker.IsComplete(condition):
+    if condition not in self._tracker or self._tracker.IsComplete(condition):
       return
 
     self._tracker.FailStage(
@@ -518,7 +517,7 @@ class ServerlessOperations(object):
       region: str, The region of the control plane if operating against
         hosted Cloud Run, else None.
       op_client: The API client for interacting with One Platform APIs. Or
-        None if interacting with Cloud Run on GKE.
+        None if interacting with Cloud Run for Anthos.
     """
     self._client = client
     self._registry = resources.REGISTRY.Clone()
@@ -547,6 +546,8 @@ class ServerlessOperations(object):
       with metrics.RecordDuration(metric_names.GET_REVISION):
         response = self._client.namespaces_revisions.Get(request)
       return revision.Revision(response, messages)
+    except api_exceptions.InvalidDataFromServerError as e:
+      serverless_exceptions.MaybeRaiseCustomFieldMismatch(e)
     except api_exceptions.HttpNotFoundError:
       return None
 
@@ -626,23 +627,32 @@ class ServerlessOperations(object):
     return serv_route.active_revisions
 
   def ListServices(self, namespace_ref):
+    """Returns all services in the namespace."""
     messages = self.messages_module
     request = messages.RunNamespacesServicesListRequest(
         parent=namespace_ref.RelativeName())
-    with metrics.RecordDuration(metric_names.LIST_SERVICES):
-      response = self._client.namespaces_services.List(request)
-    return [service.Service(item, messages) for item in response.items]
+    try:
+      with metrics.RecordDuration(metric_names.LIST_SERVICES):
+        response = self._client.namespaces_services.List(request)
+      return [service.Service(item, messages) for item in response.items]
+    except api_exceptions.InvalidDataFromServerError as e:
+      serverless_exceptions.MaybeRaiseCustomFieldMismatch(e)
 
   def ListConfigurations(self, namespace_ref):
+    """Returns all configurations in the namespace."""
     messages = self.messages_module
     request = messages.RunNamespacesConfigurationsListRequest(
         parent=namespace_ref.RelativeName())
-    with metrics.RecordDuration(metric_names.LIST_CONFIGURATIONS):
-      response = self._client.namespaces_configurations.List(request)
-    return [configuration.Configuration(item, messages)
-            for item in response.items]
+    try:
+      with metrics.RecordDuration(metric_names.LIST_CONFIGURATIONS):
+        response = self._client.namespaces_configurations.List(request)
+      return [configuration.Configuration(item, messages)
+              for item in response.items]
+    except api_exceptions.InvalidDataFromServerError as e:
+      serverless_exceptions.MaybeRaiseCustomFieldMismatch(e)
 
   def ListRoutes(self, namespace_ref):
+    """Returns all routes in the namespace."""
     messages = self.messages_module
     request = messages.RunNamespacesRoutesListRequest(
         parent=namespace_ref.RelativeName())
@@ -660,7 +670,9 @@ class ServerlessOperations(object):
       with metrics.RecordDuration(metric_names.GET_SERVICE):
         service_get_response = self._client.namespaces_services.Get(
             service_get_request)
-      return service.Service(service_get_response, messages)
+        return service.Service(service_get_response, messages)
+    except api_exceptions.InvalidDataFromServerError as e:
+      serverless_exceptions.MaybeRaiseCustomFieldMismatch(e)
     except api_exceptions.HttpNotFoundError:
       return None
 
@@ -685,6 +697,8 @@ class ServerlessOperations(object):
         configuration_get_response = self._client.namespaces_configurations.Get(
             configuration_get_request)
       return configuration.Configuration(configuration_get_response, messages)
+    except api_exceptions.InvalidDataFromServerError as e:
+      serverless_exceptions.MaybeRaiseCustomFieldMismatch(e)
     except api_exceptions.HttpNotFoundError:
       return None
 
@@ -760,8 +774,11 @@ class ServerlessOperations(object):
     request = messages.RunNamespacesRevisionsListRequest(
         parent=namespace_ref.RelativeName(),
         labelSelector='{} = {}'.format(revision.NONCE_LABEL, nonce))
-    response = self._client.namespaces_revisions.List(request)
-    return [revision.Revision(item, messages) for item in response.items]
+    try:
+      response = self._client.namespaces_revisions.List(request)
+      return [revision.Revision(item, messages) for item in response.items]
+    except api_exceptions.InvalidDataFromServerError as e:
+      serverless_exceptions.MaybeRaiseCustomFieldMismatch(e)
 
   def _GetBaseRevision(self, template, metadata, status):
     """Return a Revision for use as the "base revision" for a change.
@@ -808,9 +825,17 @@ class ServerlessOperations(object):
       base_revision_nonce = template.labels.get(revision.NONCE_LABEL, None)
       if base_revision_nonce:
         try:
-          namespace_ref = self._registry.Parse(
-              metadata.namespace,
-              collection='run.namespaces')
+          # TODO(b/148817410): Remove this when the api has been split.
+          # This try/except block is needed because the v1alpha1 and v1 run apis
+          # have different collection names for the namespaces.
+          try:
+            namespace_ref = self._registry.Parse(
+                metadata.namespace,
+                collection='run.namespaces')
+          except resources.InvalidCollectionException:
+            namespace_ref = self._registry.Parse(
+                metadata.namespace,
+                collection='run.api.v1.namespaces')
           poller = NonceBasedRevisionPoller(self, namespace_ref)
           base_revision = poller.GetResult(waiter.PollUntilDone(
               poller, base_revision_nonce,
@@ -820,8 +845,7 @@ class ServerlessOperations(object):
     # Nonce polling didn't work, because some client didn't post one or didn't
     # change one. Fall back to the (slightly racy) `latestCreatedRevisionName`.
     if not base_revision:
-      # TODO(b/117663680) Getattr -> normal access.
-      if getattr(status, 'latestCreatedRevisionName', None):
+      if status.latestCreatedRevisionName:
         # Get by latestCreatedRevisionName
         revision_ref = self._registry.Parse(
             status.latestCreatedRevisionName,
@@ -887,6 +911,8 @@ class ServerlessOperations(object):
           raw_service = self._client.namespaces_services.Create(
               serv_create_req)
         return service.Service(raw_service, messages)
+    except api_exceptions.InvalidDataFromServerError as e:
+      serverless_exceptions.MaybeRaiseCustomFieldMismatch(e)
     except api_exceptions.HttpBadRequestError as e:
       exceptions.reraise(serverless_exceptions.HttpError(e))
     except api_exceptions.HttpNotFoundError as e:
@@ -958,15 +984,38 @@ class ServerlessOperations(object):
         str(curr_generation + 1).zfill(5), name_generator.GenerateName())
     config_changes.insert(0, _NewRevisionForcingChange(revision_suffix))
 
-  def ReleaseService(self, service_ref, config_changes, tracker=None,
-                     asyn=False, allow_unauthenticated=None, for_replace=False):
+  def _BuildFromSource(self, tracker, build_messages, build_config):
+    """Build an image from source if a user specifies a source when deploying."""
+    build, build_op = submit_util.Build(build_messages, True, build_config,
+                                        hide_logs=True)
+    build_op_ref = resources.REGISTRY.ParseRelativeName(
+        build_op.name, 'cloudbuild.operations')
+    build_log_url = build.logUrl
+    tracker.StartStage(stages.BUILD_READY)
+    tracker.UpdateHeaderMessage('Building Container.')
+    tracker.UpdateStage(
+        stages.BUILD_READY, 'Logs are available at [{build_log_url}].'.format(
+            build_log_url=build_log_url))
+    return build_op_ref, build_log_url
+
+  def ReleaseService(self,
+                     service_ref,
+                     config_changes,
+                     tracker=None,
+                     asyn=False,
+                     allow_unauthenticated=None,
+                     for_replace=False,
+                     prefetch=False,
+                     build_image=None,
+                     build_pack=None,
+                     build_source=None):
     """Change the given service in prod using the given config_changes.
 
     Ensures a new revision is always created, even if the spec of the revision
     has not changed.
 
-    Arguments:
-      service_ref: Resource, the service to release
+    Args:
+      service_ref: Resource, the service to release.
       config_changes: list, objects that implement Adjust().
       tracker: StagedProgressTracker, to report on the progress of releasing.
       asyn: bool, if True, return without waiting for the service to be updated.
@@ -976,12 +1025,52 @@ class ServerlessOperations(object):
         unauthenticated access from a service.
       for_replace: bool, If the change is for a replacing the service from a
         YAML specification.
+      prefetch: the service, pre-fetched for ReleaseService. `False` indicates
+        the caller did not perform a prefetch; `None` indicates a nonexistant
+        service.
+      build_image: The build image reference to the build.
+      build_pack: The build pack reference to the build.
+      build_source: The build source reference to the build.
+    Returns:
+      service.Service, the service as returned by the server on the POST/PUT
+       request to create/update the service.
     """
     if tracker is None:
       tracker = progress_tracker.NoOpStagedProgressTracker(
           stages.ServiceStages(allow_unauthenticated is not None),
-          interruptable=True, aborted_message='aborted')
-    serv = self.GetService(service_ref)
+          interruptable=True,
+          aborted_message='aborted')
+
+    if build_source is not None:
+      tracker.StartStage(stages.UPLOAD_SOURCE)
+      tracker.UpdateHeaderMessage('Uploading sources.')
+      build_messages = cloudbuild_util.GetMessagesModule()
+      build_config = submit_util.CreateBuildConfigAlpha(
+          build_image, False, build_messages, None, None,
+          True, False, build_source, None, None, None,
+          None, None, None, build_pack, hide_logs=True)
+      tracker.CompleteStage(stages.UPLOAD_SOURCE)
+
+      build_op_ref, build_log_url = self._BuildFromSource(
+          tracker, build_messages, build_config)
+      client = cloudbuild_util.GetClientInstance()
+      poller = waiter.CloudOperationPoller(client.projects_builds,
+                                           client.operations)
+      operation = waiter.PollUntilDone(poller, build_op_ref)
+      response_dict = encoding.MessageToPyValue(operation.response)
+      if response_dict and response_dict['status'] != 'SUCCESS':
+        tracker.FailStage(
+            stages.BUILD_READY, None,
+            message='Container build failed and '
+            'logs are available at [{build_log_url}].'.format(
+                build_log_url=build_log_url))
+        return
+      else:
+        tracker.CompleteStage(stages.BUILD_READY)
+    if prefetch is None:
+      serv = None
+    else:
+      serv = prefetch or self.GetService(service_ref)
     if for_replace:
       with_image = True
     else:
@@ -993,7 +1082,7 @@ class ServerlessOperations(object):
         self._EnsureImageDigest(serv, config_changes)
     config_changes = [_SetClientNameAndVersion()] + config_changes
 
-    self._UpdateOrCreateService(
+    created_or_updated_service = self._UpdateOrCreateService(
         service_ref, config_changes, with_image, serv)
 
     if allow_unauthenticated is not None:
@@ -1025,6 +1114,7 @@ class ServerlessOperations(object):
       self.WaitForCondition(poller)
       for msg in run_condition.GetNonTerminalMessages(poller.GetConditions()):
         tracker.AddWarning(msg)
+    return created_or_updated_service
 
   def ListRevisions(self, namespace_ref, service_name,
                     limit=None, page_size=100):
@@ -1054,15 +1144,18 @@ class ServerlessOperations(object):
       # 'service-less' operation.
       request.labelSelector = 'serving.knative.dev/service = {}'.format(
           service_name)
-    for result in list_pager.YieldFromList(
-        service=self._client.namespaces_revisions,
-        request=request,
-        limit=limit,
-        batch_size=page_size,
-        current_token_attribute='continue_',
-        next_token_attribute=('metadata', 'continue_'),
-        batch_size_attribute='limit'):
-      yield revision.Revision(result, messages)
+    try:
+      for result in list_pager.YieldFromList(
+          service=self._client.namespaces_revisions,
+          request=request,
+          limit=limit,
+          batch_size=page_size,
+          current_token_attribute='continue_',
+          next_token_attribute=('metadata', 'continue_'),
+          batch_size_attribute='limit'):
+        yield revision.Revision(result, messages)
+    except api_exceptions.InvalidDataFromServerError as e:
+      serverless_exceptions.MaybeRaiseCustomFieldMismatch(e)
 
   def ListDomainMappings(self, namespace_ref):
     """List all domain mappings.
@@ -1081,12 +1174,18 @@ class ServerlessOperations(object):
     return [domain_mapping.DomainMapping(item, messages)
             for item in response.items]
 
-  def CreateDomainMapping(self, domain_mapping_ref, service_name):
+  def CreateDomainMapping(self,
+                          domain_mapping_ref,
+                          service_name,
+                          config_changes,
+                          force_override=False):
     """Create a domain mapping.
 
     Args:
       domain_mapping_ref: Resource, domainmapping resource.
       service_name: str, the service to which to map domain.
+      config_changes: list of ConfigChanger to modify the domainmapping with
+      force_override: bool, override an existing mapping of this domain.
 
     Returns:
       A domain_mapping.DomainMapping object.
@@ -1097,6 +1196,10 @@ class ServerlessOperations(object):
         self._client, domain_mapping_ref.namespacesId)
     new_mapping.name = domain_mapping_ref.domainmappingsId
     new_mapping.route_name = service_name
+    new_mapping.force_override = force_override
+
+    for config_change in config_changes:
+      new_mapping = config_change.Adjust(new_mapping)
 
     request = messages.RunNamespacesDomainmappingsCreateRequest(
         domainMapping=new_mapping.Message(),
@@ -1117,6 +1220,11 @@ class ServerlessOperations(object):
       if ready and ready.get('message'):
         message = ready['message']
       if not mapping.records:
+        if (mapping.ready_condition['reason'] ==
+            domain_mapping.MAPPING_ALREADY_EXISTS_CONDITION_REASON):
+          raise serverless_exceptions.DomainMappingAlreadyExistsError(
+              'Domain mapping to [{}] is already in use elsewhere.'.format(
+                  domain_mapping_ref.Name()))
         raise serverless_exceptions.DomainMappingCreationError(
             message or 'Could not create domain mapping.')
       if message:
