@@ -55,12 +55,24 @@ DOCUMENTATION = '''
         with_tags:
             description:
             - Include tags and associated virtual machines.
-            - Requires 'vSphere Automation SDK' and 'vCloud Suite SDK' libraries to be installed on the given controller machine.
+            - Requires 'vSphere Automation SDK' library to be installed on the given controller machine.
             - Please refer following URLs for installation steps
             - 'https://code.vmware.com/web/sdk/65/vsphere-automation-python'
-            - 'https://code.vmware.com/web/sdk/60/vcloudsuite-python'
             default: False
             type: boolean
+        properties:
+            description:
+            - Specify the list of VMware schema properties associated with the VM.
+            - These properties will be populated in hostvars of the given VM.
+            - Each value in the list specifies the path to a specific property in VM object.
+            type: list
+            default: [ 'name', 'config.cpuHotAddEnabled', 'config.cpuHotRemoveEnabled',
+                       'config.instanceUuid', 'config.hardware.numCPU', 'config.template',
+                       'config.name', 'guest.hostName', 'guest.ipAddress',
+                       'guest.guestId', 'guest.guestState', 'runtime.maxMemoryUsage',
+                       'customValue'
+                       ]
+            version_added: "2.9"
 '''
 
 EXAMPLES = '''
@@ -72,6 +84,18 @@ EXAMPLES = '''
     password: Esxi@123$%
     validate_certs: False
     with_tags: True
+
+# Gather minimum set of properties for VMware guest
+    plugin: vmware_vm_inventory
+    strict: False
+    hostname: 10.65.223.31
+    username: administrator@vsphere.local
+    password: Esxi@123$%
+    validate_certs: False
+    with_tags: False
+    properties:
+    - 'name'
+    - 'guest.ipAddress'
 '''
 
 import ssl
@@ -93,22 +117,13 @@ except ImportError:
     HAS_PYVMOMI = False
 
 try:
-    from vmware.vapi.lib.connect import get_requests_connector
-    from vmware.vapi.security.session import create_session_security_context
-    from vmware.vapi.security.user_password import create_user_password_security_context
-    from com.vmware.cis_client import Session
     from com.vmware.vapi.std_client import DynamicID
-    from com.vmware.cis.tagging_client import Tag, TagAssociation
-    HAS_VCLOUD = True
-except ImportError:
-    HAS_VCLOUD = False
-
-try:
-    from vmware.vapi.stdlib.client.factories import StubConfigurationFactory
+    from vmware.vapi.vsphere.client import create_vsphere_client
     HAS_VSPHERE = True
 except ImportError:
     HAS_VSPHERE = False
 
+
 from ansible.plugins.inventory import BaseInventoryPlugin, Cacheable
 
 
@@ -144,28 +159,13 @@ class BaseVMwareInventory:
             # Disable warning shown at stdout
             requests.packages.urllib3.disable_warnings()
 
-        vcenter_url = "https://%s/api" % self.hostname
-
-        # Get request connector
-        connector = get_requests_connector(session=session, url=vcenter_url)
-        # Create standard Configuration
-        stub_config = StubConfigurationFactory.new_std_configuration(connector)
-        # Use username and password in the security context to authenticate
-        security_context = create_user_password_security_context(self.username, self.password)
-        # Login
-        stub_config.connector.set_security_context(security_context)
-        # Create the stub for the session service and login by creating a session.
-        session_svc = Session(stub_config)
-        session_id = session_svc.create()
-
-        # After successful authentication, store the session identifier in the security
-        # context of the stub and use that for all subsequent remote requests
-        session_security_context = create_session_security_context(session_id)
-        stub_config.connector.set_security_context(session_security_context)
-
-        if stub_config is None:
+        client = create_vsphere_client(server=self.hostname,
+                                       username=self.username,
+                                       password=self.password,
+                                       session=session)
+        if client is None:
             raise AnsibleError("Failed to login to %s using %s" % (self.hostname, self.username))
-        return stub_config
+        return client
 
     def _login(self):
         """
@@ -235,11 +235,6 @@ class BaseVMwareInventory:
                                " Please refer this URL for installation steps"
                                " - https://code.vmware.com/web/sdk/65/vsphere-automation-python")
 
-        if not HAS_VCLOUD and self.with_tags:
-            raise AnsibleError("Unable to find 'vCloud Suite SDK' Python library which is required."
-                               " Please refer this URL for installation steps"
-                               " - https://code.vmware.com/web/sdk/60/vcloudsuite-python")
-
         if not all([self.hostname, self.username, self.password]):
             raise AnsibleError("Missing one of the following : hostname, username, password. Please read "
                                "the documentation for more information.")
@@ -394,8 +389,8 @@ class InventoryModule(BaseInventoryPlugin, Cacheable):
                                                            properties=['name'])
 
         if self.pyv.with_tags:
-            tag_svc = Tag(self.pyv.rest_content)
-            tag_association = TagAssociation(self.pyv.rest_content)
+            tag_svc = self.pyv.rest_content.tagging.Tag
+            tag_association = self.pyv.rest_content.tagging.TagAssociation
 
             tags_info = dict()
             tags = tag_svc.list()
@@ -428,7 +423,7 @@ class InventoryModule(BaseInventoryPlugin, Cacheable):
                     self._populate_host_properties(vm_obj, current_host)
 
                     # Only gather facts related to tag if vCloud and vSphere is installed.
-                    if HAS_VCLOUD and HAS_VSPHERE and self.pyv.with_tags:
+                    if HAS_VSPHERE and self.pyv.with_tags:
                         # Add virtual machine to appropriate tag group
                         vm_mo_id = vm_obj.obj._GetMoId()
                         vm_dynamic_id = DynamicID(type='VirtualMachine', id=vm_mo_id)
@@ -462,24 +457,9 @@ class InventoryModule(BaseInventoryPlugin, Cacheable):
 
     def _populate_host_properties(self, vm_obj, current_host):
         # Load VM properties in host_vars
-        vm_properties = [
-            'name',
-            'config.cpuHotAddEnabled',
-            'config.cpuHotRemoveEnabled',
-            'config.instanceUuid',
-            'config.hardware.numCPU',
-            'config.template',
-            'config.name',
-            'guest.hostName',
-            'guest.ipAddress',
-            'guest.guestId',
-            'guest.guestState',
-            'runtime.maxMemoryUsage',
-            'customValue',
-        ]
-        field_mgr = []
-        if self.pyv.content.customFieldsManager:
-            field_mgr = self.pyv.content.customFieldsManager.field
+        vm_properties = self.get_option('properties') or []
+
+        field_mgr = self.pyv.content.customFieldsManager.field
 
         for vm_prop in vm_properties:
             if vm_prop == 'customValue':
