@@ -30,8 +30,8 @@
 
 namespace OPNsense\Dnscryptproxy\Api;
 
-use \OPNsense\Base\ApiMutableModelControllerBase;
-use \OPNsense\Core\Backend;
+use OPNsense\Base\ApiMutableModelControllerBase;
+use OPNsense\Core\Backend;
 
 class WhitelistController extends ApiMutableModelControllerBase
 {
@@ -40,7 +40,7 @@ class WhitelistController extends ApiMutableModelControllerBase
 
     public function searchWhitelistAction()
     {
-        return $this->searchBase('whitelists.whitelist', array("enabled", "name"));
+        return $this->searchBase('whitelists.whitelist', array("enabled", "name", "description"));
     }
     public function getWhitelistAction($uuid = null)
     {
