@@ -119,7 +119,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     } elseif ($act == 'new' || $act == 'edit') {
         // edit user, load or init data
         $fieldnames = array('user_dn', 'descr', 'expires', 'scope', 'uid', 'priv', 'ipsecpsk',
-                            'lifetime', 'otp_seed', 'email', 'shell', 'comment', 'landing_page');
+                            'otp_seed', 'email', 'shell', 'comment', 'landing_page');
         if (isset($id)) {
             if (isset($a_user[$id]['authorizedkeys'])) {
                 $pconfig['authorizedkeys'] = base64_decode($a_user[$id]['authorizedkeys']);
@@ -148,7 +148,6 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             $pconfig['groups'] = null;
             $pconfig['disabled'] = false;
             $pconfig['scope'] = "user";
-            $pconfig['lifetime'] = 365;
             $pconfig['usernamefld'] = null;
             foreach ($fieldnames as $fieldname) {
                 if (!isset($pconfig[$fieldname])) {
@@ -387,6 +386,8 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             local_user_set_groups($userent, $pconfig['groups']);
             local_user_set($userent);
             write_config();
+            // XXX: signal backend that the user has changed.
+            configdp_run('auth user changed', [$userent['name']]);
 
             if (!empty($pconfig['chkNewCert'])) {
                 header(url_safe('Location: /system_certmanager.php?act=new&userid=%d', array(isset($id) ? $id : count($a_user) - 1)));
@@ -477,7 +478,7 @@ $( document ).ready(function() {
 
     // expand ssh key section on click
     $("#authorizedkeys").click(function(){
-        $(this).attr('rows', '7');
+        $(this).attr('rows', '10');
     });
 
     // import ldap users
@@ -799,16 +800,21 @@ $( document ).ready(function() {
                     <td>
                       <table class="table table-condensed">
                         <tr>
-                          <td><?=gettext("Name");?></td>
-                          <td><?=gettext("CA");?></td>
+                          <td><strong><?=gettext("Name");?></strong></td>
+                          <td><strong><?=gettext("CA");?></strong></td>
+                          <td><strong><?=gettext("Valid From");?></strong></td>
+                          <td><strong><?=gettext("Valid To");?></strong></td>
                           <td></td>
                         </tr>
 <?php
+                        $new_cert_link_suffix = "";
                         if (isset($a_user[$id]['cert']) && is_array($a_user[$id]['cert'])) :
                           $i = 0;
                           foreach ($a_user[$id]['cert'] as $certref) :
                             $cert = lookup_cert($certref);
                             $ca = lookup_ca($cert['caref']);
+                            list($cert_validfrom, $cert_validto) = cert_get_dates($cert['crt']);
+                            $new_cert_link_suffix = "&amp;method=internal&amp;caref={$cert['caref']}";
 ?>
                         <tr>
                           <td><?=htmlspecialchars($cert['descr']);?>
@@ -817,6 +823,8 @@ $( document ).ready(function() {
                           <td>
                             <?=htmlspecialchars($ca['descr']);?>
                           </td>
+                          <td><?=$cert_validfrom;?></td>
+                          <td><?=$cert_validto;?></td>
                           <td>
                             <a href="system_usermanager.php?act=expckey&amp;certid=<?=$i?>&amp;userid=<?=$id?>"
                                 class="btn btn-default btn-xs" data-toggle="tooltip" title="<?=gettext("export private key");?>">
@@ -837,8 +845,8 @@ $( document ).ready(function() {
                             endforeach;
                         endif;?>
                         <tr>
-                          <td colspan="3">
-                            <a href="system_certmanager.php?act=new&amp;userid=<?=$id?>" class="btn btn-default btn-xs"
+                          <td colspan="5">
+                            <a href="system_certmanager.php?act=new&amp;userid=<?=$id?><?=$new_cert_link_suffix;?>" class="btn btn-default btn-xs"
                                 title="<?=gettext("create or link user certificate");?>" data-toggle="tooltip">
                               <span class="fa fa-plus fa-fw"></span>
                             </a>
@@ -946,7 +954,7 @@ $( document ).ready(function() {
                   <tr>
                     <td><i class="fa fa-info-circle text-muted"></i> <?=gettext("Authorized keys");?></td>
                     <td>
-                      <textarea name="authorizedkeys" id="authorizedkeys" class="form-control" cols="65" rows="1" placeholder="<?=gettext("Paste an authorized keys file here.");?>" wrap='off'><?=$pconfig['authorizedkeys'];?></textarea>
+                      <textarea name="authorizedkeys" id="authorizedkeys" style="max-width: inherit;" class="form-control" cols="65" rows="1" placeholder="<?=gettext("Paste an authorized keys file here.");?>" wrap='off'><?=$pconfig['authorizedkeys'];?></textarea>
                     </td>
                   </tr>
                   <tr id="ipsecpskrow">
